//
//  DTKViewController.m
//  WebRadioPlayer
//
//  Created by Chris Adamson on 4/19/13.
//  Copyright (c) 2013 Chris Adamson. All rights reserved.
//

#import "DTKViewController.h"
#import <AudioToolbox/AudioToolbox.h>

#define STATION_URL_STRING @"http://1661.live.streamtheworld.com:80/CBC_R3_WEB_SC"

@interface DTKViewController ()
@property (weak, nonatomic) IBOutlet UILabel *stationLabel;
@property (assign) AudioFileStreamID audioFileStream;
@property (assign) AudioStreamBasicDescription streamFormat;
@property (assign) AudioQueueRef audioQueue;
@property (assign) UInt32 totalPacketCount;
@property (assign) BOOL queueStarted;

@end

@implementation DTKViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
	NSURL *stationURL = [NSURL URLWithString:STATION_URL_STRING];
	NSURLRequest *request= [NSURLRequest requestWithURL:stationURL];
	[NSURLConnection connectionWithRequest:request delegate:self];
	self.stationLabel.text = STATION_URL_STRING;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark NSURLConnection delegate
-(void) connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
//	NSLog (@"received %d", [data length]);
	OSStatus err = noErr;
	err = AudioFileStreamParseBytes(self.audioFileStream,
									[data length],
									[data bytes],
									0);
	
}

-(void) connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	OSStatus err = noErr;
	err = AudioFileStreamOpen((__bridge void *)(self),
							  propertyProc,
							  packetsProc,
							  kAudioFileMP3Type,
							  &_audioFileStream);
	if (err) { NSLog (@"couldn't open file stream"); }
	
}

#pragma mark AudioFileStream callbacks
void propertyProc (
				   void                        *inClientData,
				   AudioFileStreamID           inAudioFileStream,
				   AudioFileStreamPropertyID   inPropertyID,
				   UInt32                      *ioFlags
				   ) {
	NSLog(@"property %ld", inPropertyID);
	DTKViewController *vc = (__bridge DTKViewController*) inClientData;
	if (inPropertyID == kAudioFileStreamProperty_DataFormat) {
		OSStatus err = noErr;
		AudioStreamBasicDescription dataFormat;
		UInt32 propSize = sizeof(dataFormat);
		
		err = AudioFileStreamGetProperty(inAudioFileStream,
										 kAudioFileStreamProperty_DataFormat,
										 &propSize,
										 &dataFormat);
		if (!err) {
			vc.streamFormat = dataFormat;
		}
	} else if (inPropertyID == kAudioFileStreamProperty_ReadyToProducePackets) {
		AudioStreamBasicDescription dataFormat = vc.streamFormat;
		AudioQueueRef audioQueue;
		OSStatus err = AudioQueueNewOutput(&dataFormat,
										   queueCallbackProc,
										   (__bridge void *)(vc),
										   NULL,
										   NULL,
										   0,
										   &audioQueue);
		if (!err) {
			vc.audioQueue = audioQueue;
		}
		
		
	}
	
	
}

void packetsProc (
				  void                          *inClientData,
				  UInt32                        inNumberBytes,
				  UInt32                        inNumberPackets,
				  const void                    *inInputData,
				  AudioStreamPacketDescription  *inPacketDescriptions
				  ) {
	DTKViewController *vc = (__bridge DTKViewController*) inClientData;
	AudioQueueBufferRef buffer;
	OSStatus err = AudioQueueAllocateBuffer(vc.audioQueue,
											inNumberBytes,
											&buffer);
	if (err) { NSLog (@"couldn't allocate buffer"); return; }
	buffer->mAudioDataByteSize = inNumberBytes;
	memcpy(buffer->mAudioData, inInputData, inNumberBytes);
	
	err = AudioQueueEnqueueBuffer(vc.audioQueue,
								  buffer,
								  inNumberPackets,
								  inPacketDescriptions);
	if(err) { NSLog (@"couldn't enqueue buffer"); }
	vc.totalPacketCount += inNumberPackets;
	if (!vc.queueStarted && vc.totalPacketCount > 100) {
		err = AudioQueueStart(vc.audioQueue,
							  NULL);
		if (err) { NSLog (@"couldn't start queue"); }
	}
	
}

#pragma mark Audio Queue callbacks
void queueCallbackProc (
						void                 *inUserData,
						AudioQueueRef        inAQ,
						AudioQueueBufferRef  inBuffer
						) {
	OSStatus err = AudioQueueFreeBuffer(inAQ,
										inBuffer);
	if (err) { NSLog(@"couldn't free buffer"); }
}


@end
